@@ -105,14 +105,32 @@ class APIPolicyBase(object):
 
 
 class CacheImageAPIPolicy(APIPolicyBase):
+    def __init__(self, context, image=None, policy_str=None,
+                 target=None, enforcer=None):
+        self._context = context
+        target = {}
+        self._image = image
+        if self._image:
+            target = policy.ImageTarget(self._image)
+
+        self._target = target
+        self.enforcer = enforcer or policy.Enforcer()
+        self.policy_str = policy_str
+        super(CacheImageAPIPolicy, self).__init__(context, target, enforcer)
+
+    def manage_image_cache(self):
+        self._enforce(self.policy_str)
+
+
+class DiscoveryAPIPolicy(APIPolicyBase):
     def __init__(self, context, target=None, enforcer=None):
         self._context = context
         self._target = target or {}
         self.enforcer = enforcer or policy.Enforcer()
-        super(CacheImageAPIPolicy, self).__init__(context, target, enforcer)
+        super(DiscoveryAPIPolicy, self).__init__(context, target, enforcer)
 
-    def manage_image_cache(self):
-        self._enforce('manage_image_cache')
+    def stores_info_detail(self):
+        self._enforce('stores_info_detail')
 
 
 class ImageAPIPolicy(APIPolicyBase):
@@ -207,7 +225,8 @@ class ImageAPIPolicy(APIPolicyBase):
         self._enforce('delete_image_location')
         # TODO(danms): Remove this legacy fallback when secure RBAC
         # replaces the legacy policy.
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_is_image_mutable(self._context, self._image)
 
     def get_image_location(self):
@@ -229,7 +248,8 @@ class ImageAPIPolicy(APIPolicyBase):
                 raise
         if 'visibility' in self._target:
             self._enforce_visibility(self._target['visibility'])
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_admin_or_same_owner(self._context, self._target)
 
     def get_image(self):
@@ -242,14 +262,16 @@ class ImageAPIPolicy(APIPolicyBase):
         self._enforce('delete_image')
         # TODO(danms): Remove this legacy fallback when secure RBAC
         # replaces the legacy policy.
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_is_image_mutable(self._context, self._image)
 
     def upload_image(self):
         self._enforce('upload_image')
         # TODO(danms): Remove this legacy fallback when secure RBAC
         # replaces the legacy policy.
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_is_image_mutable(self._context, self._image)
 
     def download_image(self):
@@ -259,21 +281,24 @@ class ImageAPIPolicy(APIPolicyBase):
         self._enforce('modify_image')
         # TODO(danms): Remove this legacy fallback when secure RBAC
         # replaces the legacy policy.
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_is_image_mutable(self._context, self._image)
 
     def deactivate_image(self):
         self._enforce('deactivate')
         # TODO(danms): Remove this legacy fallback when secure RBAC
         # replaces the legacy policy.
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_is_image_mutable(self._context, self._image)
 
     def reactivate_image(self):
         self._enforce('reactivate')
         # TODO(danms): Remove this legacy fallback when secure RBAC
         # replaces the legacy policy.
-        if not CONF.enforce_secure_rbac:
+        if not (CONF.oslo_policy.enforce_new_defaults or
+                CONF.oslo_policy.enforce_scope):
             check_is_image_mutable(self._context, self._image)
 
     def copy_image(self):
