@@ -17,19 +17,19 @@ SQLAlchemy models for glance metadata schema
 """
 
 from oslo_db.sqlalchemy import models
-from oslo_utils import timeutils
 from sqlalchemy import Boolean
 from sqlalchemy import Column
 from sqlalchemy import DateTime
-from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import ForeignKey
 from sqlalchemy import Index
 from sqlalchemy import Integer
+from sqlalchemy.orm import declarative_base
 from sqlalchemy.orm import relationship
 from sqlalchemy import String
 from sqlalchemy import Text
 from sqlalchemy import UniqueConstraint
 
+from glance.common import timeutils
 from glance.db.sqlalchemy.models import JSONEncodedDict
 
 
@@ -56,7 +56,7 @@ class GlanceMetadefBase(models.TimestampMixin):
     created_at = Column(DateTime, default=lambda: timeutils.utcnow(),
                         nullable=False)
     # TODO(wko): Column `updated_at` have no default value in
-    #            openstack common code. We should decide, is this value
+    #            OpenStack common code. We should decide, is this value
     #            required and make changes in oslo (if required) or
     #            in glance (if not).
     updated_at = Column(DateTime, default=lambda: timeutils.utcnow(),
