@@ -18,8 +18,8 @@ from oslo_policy import policy
 discovery_policies = [
     policy.DocumentedRuleDefault(
         name="stores_info_detail",
-        check_str='role:admin',
-        scope_types=['system', 'project'],
+        check_str='rule:context_is_admin',
+        scope_types=['project'],
         description='Expose store specific information',
         operations=[
             {'path': '/v2/info/stores/detail',
