@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import http.client as http
+
 from oslo_utils import encodeutils
-import six
 
 from glance.common import exception
 from glance.tests import utils as test_utils
@@ -38,15 +39,15 @@ class GlanceExceptionTestCase(test_utils.BaseTestCase):
         class FakeGlanceException(exception.GlanceException):
             message = "default message: %(code)s"
 
-        exc = FakeGlanceException(code=500)
+        exc = FakeGlanceException(code=int(http.INTERNAL_SERVER_ERROR))
         self.assertEqual("default message: 500",
                          encodeutils.exception_to_unicode(exc))
 
     def test_specified_error_msg_with_kwargs(self):
-        msg = exception.GlanceException('test: %(code)s', code=500)
+        msg = exception.GlanceException('test: %(code)s',
+                                        code=int(http.INTERNAL_SERVER_ERROR))
         self.assertIn('test: 500', encodeutils.exception_to_unicode(msg))
 
     def test_non_unicode_error_msg(self):
-        exc = exception.GlanceException(str('test'))
-        self.assertIsInstance(encodeutils.exception_to_unicode(exc),
-                              six.text_type)
+        exc = exception.GlanceException('test')
+        self.assertIsInstance(encodeutils.exception_to_unicode(exc), str)
