@@ -515,6 +515,18 @@ class TestMetadefRepo(test_utils.BaseTestCase):
         tag_names = set([t.name for t in tags])
         self.assertEqual(set([TAG3, TAG4, TAG5]), tag_names)
 
+    def test_add_tags_with_append_true(self):
+        tags = self.tag_repo.list(filters={'namespace': NAMESPACE1})
+        tag_names = set([t.name for t in tags])
+        self.assertEqual(set([TAG1, TAG2, TAG3]), tag_names)
+
+        tags = _db_tags_fixture([TAG4, TAG5])
+        self.db.metadef_tag_create_tags(self.context, NAMESPACE1, tags,
+                                        can_append=True)
+        tags = self.tag_repo.list(filters={'namespace': NAMESPACE1})
+        tag_names = set([t.name for t in tags])
+        self.assertEqual(set([TAG1, TAG2, TAG3, TAG4, TAG5]), tag_names)
+
     def test_add_duplicate_tags_with_pre_existing_tags(self):
         tags = self.tag_repo.list(filters={'namespace': NAMESPACE1})
         tag_names = set([t.name for t in tags])
