@@ -32,6 +32,7 @@ from unittest import mock
 
 from alembic import command as alembic_command
 import fixtures
+from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_config import fixture as cfg_fixture
 from oslo_log.fixture import logging_error as log_fixture
@@ -41,6 +42,7 @@ from oslo_utils import units
 import testtools
 import webob
 
+from glance.api.v2 import cached_images
 from glance.common import config
 from glance.common import exception
 from glance.common import property_utils
@@ -85,6 +87,9 @@ class BaseTestCase(testtools.TestCase):
         self.test_dir2 = self.useFixture(fixtures.TempDir()).path
         self.conf_dir = os.path.join(self.test_dir, 'etc')
         utils.safe_mkdirs(self.conf_dir)
+        self.lock_dir = os.path.join(self.test_dir, 'locks')
+        utils.safe_mkdirs(self.lock_dir)
+        lockutils.set_defaults(self.lock_dir)
         self.set_policy()
 
         # Limit the amount of DeprecationWarning messages in the unit test logs
@@ -94,6 +99,10 @@ class BaseTestCase(testtools.TestCase):
         self.useFixture(log_fixture.get_logging_handle_error_fixture())
         self.useFixture(glance_fixtures.StandardLogging())
 
+        if cached_images.WORKER:
+            cached_images.WORKER.terminate()
+            cached_images.WORKER = None
+
     def set_policy(self):
         conf_file = "policy.yaml"
         self.policy_file = self._copy_data_file(conf_file, self.conf_dir)
@@ -314,16 +323,16 @@ def wait_for_fork(pid,
                     pass
                 elif not nice_timer.expired():
                     # Politely ask the process to GTFO
-                    LOG.warning('Killing child %i with SIGTERM' % pid)
+                    LOG.warning('Killing child %i with SIGTERM', pid)
                     os.kill(pid, signal.SIGTERM)
                 else:
                     # No more Mr. Nice Guy
-                    LOG.warning('Killing child %i with SIGKILL' % pid)
+                    LOG.warning('Killing child %i with SIGKILL', pid)
                     os.kill(pid, signal.SIGKILL)
                     expected_exitcode = signal.SIGKILL
                 time.sleep(1)
                 continue
-            LOG.info('waitpid(%i) returned %i,%i' % (pid, cpid, rc))
+            LOG.info('waitpid(%i) returned %i,%i', pid, cpid, rc)
             if rc != expected_exitcode:
                 raise RuntimeError('The exit code %d is not %d'
                                    % (rc, expected_exitcode))
@@ -332,7 +341,7 @@ def wait_for_fork(pid,
             # Nothing to wait for
             return 0
         except Exception as e:
-            LOG.error('Got wait error: %s' % e)
+            LOG.error('Got wait error: %s', e)
             if raise_error:
                 raise
 
@@ -697,12 +706,6 @@ def db_sync(version='heads', engine=None):
     alembic_command.upgrade(alembic_config, version)
 
 
-def is_sqlite_version_prior_to(major, minor):
-    import sqlite3
-    tup = sqlite3.sqlite_version_info
-    return tup[0] < major or (tup[0] == major and tup[1] < minor)
-
-
 def start_standalone_http_server():
     def _get_http_handler_class():
         class StaticHTTPRequestHandler(http.server.BaseHTTPRequestHandler):
